function [a,b,c,d,rcon]=abcdcoeff(E,A,B,C)
   
% Name
% This function find the abcd coefficients of Full-Order Darouach Observer of 1995 paper [1] 
%
% Calling Sequence
%  [a,b,c,d,rcon]=abcdcoeff(E,A,B,C);
%   
%     
% Parameters
% E, A, B,C:
% Continuos matrices of descriptor system.
% rcon:
% Desired poles of the observer
% a, b, c and d:
% Coefficients
% 
% 
%  Description
% 
%    Given a Descriptor system in the form
%     dxEdt=Ax+Bu
%      y   =Cx+Du
% if rank[E* ; C]=n, then exist coefficients a,b, c and d such that (for details consult [1])
%    aE  +  bC=In
%    cE  +  dC=0    
% 
% 
% Examples
% 
%    Descriptor system
% E=[1 0 0 0 ; 0 1 0 0; 0 0 0 0; 0 0 0 0];
% A=[0 0 1 0 ; 1 0 0 0; -1 0 0 1; 0 1 1 1];
% B=[0 0 0 -1]';
% C=[0 1 0 0];
%  Observer Gains
% poles=[-4,-3,-2,-1] % Desired poles 
% 
%  [a,b,c,d,rcon]=abcdcoeff(E,A,B,C); % a, b, c and d coefficients
% 
% [N,K,L1,L2,G,test]=darobsv95(E,A,B,C,poles) % Darouach observer
% 
%   
% See also
%   darobsv95 redobsv96  
% Author
% Francisco-Ronay López-Estrada (ronaystein@gmail.com)
% Bibliography
% [1] M. Darouach and M. Boutayeb, (1995). Design of Observer for Descriptor Form. IEEE Transactions on Automatic Control. Vol. 70, pp. 1323-1327
   

[Ee, Ae, Ae1, Be, Be1, Ce]=qrrse(E,A,B,C);
  %% proving the existence of the transformation Eq. 5.1-5.2
  % compute the dimentions
    m=size(A,1);
    n=size(A,2);
    p=size(C,1);
    r=rank(E);
    if rank([Ee;Ce])~=n % eq. 5.2
    warning('The system does not meet the observability condition')
    end
% 2do step finding the matriz
% $svd\begin{bmatrix} E \cr C \end{bmatrix}= \begin{bmatrix} \sum  \cr 0 \end{bmatrix}$

[U,Sv,V]=svd([Ee;Ce]);
S=Sv(1:size(Ee,2),:);

% Then $\begin{bmatrix} a & b \cr c & d\end{bmatrix}=\begin{bmatrix} VS^{-1} & 0 \cr 0 & I_{m+p-n}\end{bmatrix} U^T$


abcd=[V*inv(S) zeros(size(V,1),m+p-n);zeros(size(V,1),size(V,2)) eye(size(V,1),m+p-n) ]*U';
    
a=abcd(1:m, 1:r)    ;

b=abcd(1:m,r+1:size(abcd,2) ) ;   
c=abcd(m+1:m+p,1:r);
d=abcd(m+1:m+p, size(c,2) + 1:size(abcd,2));

%optinally you can select the comprobation of 
% %               $aE+bC=I$ 
    %               $cE+dC=0$
   %rcon 
    rcon=[a*Ee+b*Ce; c*Ee+d*Ce];
                
