function [N,K,L1,L2,G,test]=darobsv95(dsys,poles)
       
%This function find the gains of Full-Order Darouach Observer
% of 1995 paper [1]  
% 
% Calling Sequence
% [N,K,L1,L2,G,test]=darobsv95(dsys,poles)
%      
% Parameters
% E, A, B,C:
% Continuos matrices of descriptor system. E singular
% Poles:
% Desired poles of the observer
% N, K, L1, L2, G:
% Gains matrices of Darouach Observer
% test:
% test if the observer condition is true
% 
% 
% Description
% 
%    Given a Descriptor system in the form
%     dxEdt=Ax+Bu
%      y   =Cx+Du
% if the conditions in [1] are satisfied, then exist a full order observer
%    dzdt=Nz +L1y + L2y +Gu
%    xhat=z +by +Kdy
% where b and d are founded with abccoeff function
%      
% Examples
% 
%   % Descriptor system
% E=[1 0 0 0 ; 0 1 0 0; 0 0 0 0; 0 0 0 0];
% A=[0 0 1 0 ; 1 0 0 0; -1 0 0 1; 0 1 1 1];
% B=[0 0 0 -1]';
% C=[0 1 0 0];
% % Observer Gains
% poles=[-4,-3,-2,-1] % Desired poles 
% dsys=dsystem(E,A,B,C)
% [N,K,L1,L2,G,test]=darobsv95(dsys,poles)
% 
%   
% See also
%   abcdcoeff dobsv  
% 
% Authors
% Francisco Ronay LÓPEZ ESTRADA (ronaystein@gmail.com)
%D. THEILLIOL
%C. M. ASTORGA-ZARAGOZA
%J. C. PONSART 
% 
% Bibliography
% [1] M. Darouach and M. Boutayeb, (1995). Design of Observer for Descriptor Form. IEEE Transactions on Automatic Control. Vol. 70, pp. 1323-1327
[E,A,B,C]=dsys2ss(dsys);
[nC,Co_mat,nR,Ro_mat,nI,Io_mat]=dobsv(E,A,C);
if nI.rank~=(size(E,1)+rank(E))
    error('The system is not I-Observable')
    return;
end
    
% qr r.s.e
[Ee, Ae, Ae1, Be, Be1, Ce]=qrrse(E,A,B,C);
 
 % a,b,c,d coefficients
[a,b,c,d,rcon]=abcdcoeff(E,A,B,C);
% stability matrix
 N=diag(poles)
aA=a*Ae
cA=[c*Ae;Ce]
% solve the LME for $Ks=[K -L_2 ]$
% $N=aA+Ks\begin{bmatrix}cA \\ C\end{bmatrix}$

if rank([aA;cA])~=size(E,1)
   error('The pair (aA, [cA;C]) is not detectable')
   
end
 Ks=sdpvar(size(cA,2),size(cA,1),'full');
 
 constrains=[(aA+Ks*cA-N)==0];
ops = sdpsettings('solver','lmilab','verbose',0);
solvesdp(constrains);
Ks=double(Ks);
 
 % K gain
 n1=size(d,1);
 n2=size(Ks,2);
 K=clean(Ks(:,1:n1));
 % $L_2$ gain
L2=clean(-Ks(:,n1+1:n2));
% $L_1$ gain
L1=clean(N*(b+K*d));
% $G$ gain
G=clean((a+K*c)*Be);
test=clean(aA+Ks*cA);

