function [N,G,L,Ti,H2,Phi]=lpvpiuiobsv(dsys,alpha, solv)
% Given a Polytopic Linear Parameter Varying Descriptor System, 
% this function computes the gains of a proportional-integral unknown input observer[1] 
% Calling Sequence
% 
%   [N,G1,L,H2,B1i]=lpvpiuiobsv(dsys)
%   [N,G1,L,H2,B1i]=lpvpiuiobsv(dsys,alpha) % default alpha=1
%   [N,G1,L,H2,B1i]=lpvpiuiobsv(dsys,alpha,solv) %default solv='lmilab'
%     
% Parameters
% dsys: descriptor system, where E, A, B, C, R:
% Continuos matrices of descriptor system. E singular. Where A=A(theta), B=B(theta), R= R(theta) and C=C. Theta is varying parameter. 
% alpha:
% For the stabilize the observer in a stable LMI region (see [1])
% N,G,L,Ti,H2,Phi:
% Gains matrices of the observer. This is obtained like N(:,:,j) where (:,:,j) is for the 1,2... j model of the polytopic observer
% Bo1
% Auxiliar matrices for simulation, this can be obtained from qrrse transformation
% Description
% 
%   Given a Descriptor system in the form
%        M
% dxEdt= Σmu(theta)[A(theta) x(t) + B(theta) u(t) +R(theta) d (t)]
%        i=1
%      y   =C x(t) + D u(t) 
%  This compute the observer gains for 
%          M
%  dz/dt= Σmu(theta) [Niz(t) + Giu(t) + Liy(t)+Tide(t)]
%         i=1
%  xe(t)=  z(t) + H2y(t) 
%         M
%  de(t)= Σmu(theta) [z(t) +H2(yo(t))]
%         i=1
% where yo(t) results from QR r.s.e transformation
%  yo(t)=[-B1u(t); y(t)];  y(t)= C x(t)
% alpha put the observer in the stability LMI region
%      
% Examples
 % parameter varying
%   clear
%   theta1=[-0.05 0.05]; %[ ]
%   theta2=[-0.1 0.1 ];
%   
%   % genering a matrix with the two parameter varying
%   n=0;
%   for k=1:2
%       for j=1:2
%           n=n+1;
%           theta(n,:)=[theta1(k) theta2(j) ];
%       end
%   end
%   
%   % descriptor system, constant matrices
%   E=[1 0 0 0; 0 1 0 0; 0 0 0 0; 0 0 0 0];
%   
% C=[1 0 1 0; 0 1 0 1; 0 0 1 1];;
%     
%   
%   % Building the parameter varying matrices
% 
%       
% for k=1:4
%     
%     A(:,:,k)=[-1.75+theta(k,2) 1 0 0; -1 -1+theta(k,1) 0 0; -1.8 -1 -0.75+theta(k,1) 0; -1 0 0 -1+theta(k,2)];
%     
%     B(:,:,k)=[1+theta(k,1) 1;1 0.5+theta(k,2); 1 0 ;theta(k,2) 0];
%     
%     R(:,:,k)=[0 0.6+theta(k,1) 0 1]';
%     %%eq 18-21
%   
% 
%     
% 
%   end
%   %% descriptor system
%   dsys=dsystem(E,A,B,C,R) 
%   alpha=3;
%   
%   % computing the gains of the observer
%  [N,G,L,Ti,H2,Phi]=lpvpiuiobsv(dsys,alpha,'sedumi')
%   % subindex (:,:,j) indicates for the 1,2,.. jth model
%   
% See also
%   piuiobsv darobsv96  
% Author
% Francisco-Ronay López-Estrada (ronaystein@gmail.com)
% Bibliography
% [1] ﻿M., Hamdi., Rodrigues, M., Mechmeche, C., Theilliol, D., Braiek, N. B., & Tunisie, E. P. D. (2009). State Estimation for Polytopic LPV Descriptor Systems : Application to Fault Diagnosis.
% Convergence (pp. 438-443).
[E,A,B,C,R]=dsys2ss(dsys);

in=nargin();
if in<2 
    solv='lmilab'
    alpha=1;
    
elseif in<3
solv='lmilab' % default solver for the LMI
end

n=size(E,1);
  m=size(C,1);
  
%% cheking the feasibility of the observer
for j=1:size(A,3)
 [nC,Co_mat,nR,Ro_mat,nI,I_mat]=dobsv(E,A(:,:,j),C);
if rank(C)==m
    if nR.Obsv=='T'
%         if nI.Obsv=='T'
%         end
    end
else
    error ('The system does not meet the conditions of R or I Observability ');
end
end

EC=pinv([E;C]);
H1=EC(:,1:n);
H2=EC(:,n+1:end);

for j=1:size(A,3)
    %%eq 18-21
    Ti(:,:,j)=H1*R(:,:,j);
 
    G(:,:,j)=H1*B(:,:,j);
    %%Eq 32+
    aux=[H1*A(:,:,j)  Ti(:,:,j)];
    l=size(aux,2);
    Ais(:,:,j)=[aux; zeros(l-size(aux,1),l)];
Cs=[C zeros(size(C,1),l-size(aux,1))];

end


 yalmip ('clear')
 Q=sdpvar(l,l,'symmetric');
Wi=sdpvar(l,size(Cs,1),size(A,3),'full');

% %%eq. 29
%  
F=[Q>0]
for j=1:size(A,3)
F=[F, Ais(:,:,j)'*Q+Q*Ais(:,:,j)-Cs'*Wi(:,:,j)'-Wi(:,:,j)*Cs+2*alpha*Q<0  ]
end
% F=[F, Ais(:,:,4)'*Q+Q*Ais(:,:,4)-Cs'*Wi(:,:,4)'-Wi(:,:,4)*Cs+2*alpha*Q<0  ]

ops = sdpsettings('solver',solv,'verbose',0);
solvesdp(F,-trace(Q),ops)
Q=double(Q);



for j=1:size(A,3)
W(:,:,j)=double(Wi(:,:,j));
    Kis=inv(Q)*W(:,:,j);
% load Kis.mat;
K(:,:,j)=Kis(1:n,:,:);
Phi(:,:,j)=Kis(n+1,:,:);
N(:,:,j)=H1*A(:,:,j)-K(:,:,j)*C;
L(:,:,j)=K(:,:,j)+N(:,:,j)*H2;

end
