function [N,G1,L,H2,B1i]=lpvpuiobsv(dsys,alpha,solv)
% lpvpuiobsv Given a Polytopic Linear Parameter Varying Descriptor System, this function computes the gains of a full order observer with unknown inputs [1] 
% Calling Sequence
% 
%   [N,G1,L,H2,B1i]=lpvpuiobsv(dsys)
%   [N,G1,L,H2,B1i]=lpvpuiobsv(dsys,alpha) % default alpha=1
%   [N,G1,L,H2,B1i]=lpvpuiobsv(dsys,alpha,solv) %default solv='lmilab'
%     
% Parameters
% dsys: descriptor system, where E, A, B, C, R:
% Continuos matrices of descriptor system. E singular. Where A=A(theta), B=B(theta), R= R(theta) and C=C. Theta is varying parameter. 
% alpha:
% For the stabilize the observer in a stable LMI region (see [1])
% N, G1 L, H2:
% Gains matrices of the observer. This is obtained like N(:,:,j) where (:,:,j) is for the 1,2... j model of the polytopic observer
% Bo1
% Auxiliar matrices for simulation, this can be obtained from qrrse transformation
% Description
% 
%   Given a Descriptor system in the form
%        M
% dxEdt= Σmu(theta)[A(theta) x(t) + B(theta) u(t) +R(theta) d (t)]
%        i=1
%      y   =C x(t) + D u(t) 
%  This compute the observer gains for 
%          M
%  dz/dt= Σmu(theta) [Nz(t) + G1u(t) + L(yo(t))]
%         i=1
%         M
%  xe(t)= Σmu(theta) [z(t) +H2(yo(t))]
%         i=1
% where yo(t) results from QR r.s.e transformation
%  yo(t)=[-B1u(t); y(t)];  y(t)= C x(t)
% alpha put the observer in the stability LMI region
%      
% Examples
% % parameter varying
% clear
% theta1=[-0.05 0.05]; %[ ]
% theta2=[-0.1 0.1 ];
% 
% % genering a matrix with the two parameter varying
% n=0;
% for k=1:2
%     for j=1:2
%         n=n+1;
%         theta(n,:)=[theta1(k) theta2(j) ];
%     end
% end
% 
% % descriptor system, constant matrices
% E=[1 0 0 0; 0 1 0 0; 0 0 0 0; 0 0 0 0];
% 
% C=[1 0 0 0; 0 1 0 1];;
% 
% % Building the parameter varying matrices
% for j=1:4
%     
% A(:,:,j)=[0 1 0 0; -1  theta(j,1) 0 0; 0 -1 -1+theta(j,1) 0; 0 0 0 -1+theta(j,2)];
%     
% B(:,:,j)=[1+theta(j,1) 1;1 theta(j,2); 0 0 ;0.5-theta(j,2) 0];
%     
% R(:,:,j)=[1 theta(j,2) 0 0]';
% end
% %% descriptor system
% dsys=dsystem(E,A,B,C,R) 
% alpha=3;
% 
% % computing the gains of the observer
% [N,G1,L,H2,B1i]=lpvpuiobsv(dsys,alpha)
% % subindex (:,:,j) indicates for the 1,2,.. jth model
%   
% See also
%   piuiobsv darobsv96  
% Author
% Francisco-Ronay López-Estrada (ronaystein@gmail.com)
% Bibliography
% [1] ﻿M., Hamdi., Rodrigues, M., Mechmeche, C., Theilliol, D., Braiek, N. B., & Tunisie, E. P. D. (2009). State Estimation for Polytopic LPV Descriptor Systems : Application to Fault Diagnosis. Convergence (pp. 438-443).
[E,A,B,C,R]=dsys2ss(dsys);

in=nargin;
if nargin==1
    alpha=1;
    solv='lmilab'
elseif in==2
    solv='lmilab'
end
    

for j=1:size(A,3)

Io_mat=[E A(:,:,j); zeros(size(E,1),size(E,1)) E; zeros(size(C,1),size(C,2)) C]; %% Observability
if rank(Io_mat)~=size(E,1)+rank(E)
    
    error('The system is not I-Observable');
    return
end
end

% H1=list(); H2=list();
% N=list(); L=list(); G1=list();
% K=list();B1i=list();
%

n=size(E,1);
m=size(C,1)
r=rank(E);
p=size(B,2);
q=size(R,2);


%
for j=1:size(A,3)
    [Eo, Ao, Ao1, Bo, Bo1, Co, Ro,Ro0]=qrrse(E,A(:,:,j),B(:,:,j),C,R(:,:,j));
    %/ observability condition
     nIm=[Ao' Eo' Co'];
 nIma=[nIm;Eo' zeros(size(Eo,2),size(nIm,2)-size(Eo,1))]; 
 if rank(nIma)~=size(E,1)+rank(Eo)
     if rank([Eo-Ao;Co])~=size(E,1)
     error('The sistem is not I-observable')
     return
 end

 end
%     disp(rank(nIma))
    a=size(Ao,1);
%Eei(:,:,j)=Ee;
 Aoi(:,:,j)=Ao; Ao1i(:,:,j)=Ao1; Boi(:,:,j)=Bo; B1i(:,:,j)=Bo1;
  Coi(:,:,j)=Co; Roi(:,:,j)=Ro;
  % Eq. 34 
  %H12(:,:,j)=[eye(n,n) zeros(n,q)]**pinv([Eo  Roi(:,:,j); Coi(:,:,j) zeros(size(Coi(:,:,j),1),size(Roi(:,:,j),2))]);
% pause

   H12=[eye(n,n) zeros(n,q)]*pinv([Eo  Roi(:,:,j); Coi(:,:,j) zeros(size(Coi(:,:,j),1),size(Roi(:,:,j),2))]);
   H1(:,:,j)=clean(H12(:,1:a));
  H2(:,:,j)=clean(H12(:,a+1:size(H12,2)));
  
end

 

%% LMI solver 
 yalmip ('clear')
 ki=size(H1(:,:,1),1)
 l=size(Coi(:,:,1),1)
 
X=sdpvar(ki,ki,'symmetric');
W=sdpvar(ki,l,size(A,3),'full');

%%eq. 29
F=[X>0]
for j=1:size(A,3)
F=[F,((H1(:,:,j)*Aoi(:,:,j))'*X+X*(H1(:,:,j)*Aoi(:,:,j))+W(:,:,j)*Coi(:,:,j)+Coi(:,:,j)'*W(:,:,j)'+2*alpha*X)<0];
end
% F=[F,((H1(:,:,4)*Aoi(:,:,4))'*X+X*(H1(:,:,4)*Aoi(:,:,4))+W(:,:,4)*Coi(:,:,4)+Coi(:,:,4)'*W(:,:,4)'+2*alpha*X)<0];
ops = sdpsettings('solver',solv,'verbose',0);
solvesdp(F,-trace(X),ops)
X=double(X);
W=double(W);

%%
for j=1:size(A,3)
   
    K(:,:,j)=inv(X)*W(:,:,j);
%pause
%bserver parameters Eq (31-34)
N(:,:,j)=H1(:,:,j)*Aoi(:,:,j)+K(:,:,j)*Coi(:,:,j);
L(:,:,j)=clean(N(:,:,j)*H2(:,:,j)-K(:,:,j));
G1(:,:,j)=H1(:,:,j)*Boi(:,:,j);
%pause
end
