function [musa]=lpvweig4(Theta,vert)
% This function generates wieighting function for a Polytopic Linear Parameter Varying Descriptor System of four vertices 
% Calling Sequence
% 
%   [mus]=lpvweig4(Theta,vert)  
% Parameters
% Theta:
% Is the vector of measured parameter. This could be an scalar or a vector. E.g. Theta=[Theta1 Theta2]= [0.04 0.1] 
% vert:
% Are the maximun and minimun limits of the parameters. E.g. vert=[minTheta1 maxTheta1; minTheta2 maxTheta2]=[-0.05 0.05; -0.1 0.1]. Usuallly it is constant.
% mus:
% It is the vector of weightings function. mus=[mu1 mu2 mu3 mu4 sum(mu)]. sum(mu) =1 for all time
% Description
% 
%   Given a polytopic LPV Descriptor system in the form
%        M
% dxEdt= Σmu(theta)[A(theta) x(t) + B(theta) u(t) +R(theta) d (t)]
%        i=1
%      y   =C x(t) + D u(t) 
% 
% Where theta is the parameter varying and it is bounde by four vertices.
% The function builds the weighting function  Σmu(theta) given the vertices and the measured parameters theta.
% 
% Examples
% % parameter varying
% clear;
% theta1=[-0.05 0.05]; %[ ]
% theta2=[-0.1 0.1 ];
% vert=[theta1; theta2] % vertices of the polytope
% 
% % for one vector
% Thetas=[0.04  -0.1]
% [musa]=lpvweig4(Thetas,vert)
% 
% % given a vector of measured parameters
% % this it is very useful for simulation
% Theta1=idinput(101,'sine')/20;
% Theta2=idinput(101,'sine')/10;
% Thetas=[Theta1 Theta2];
% [musa]=lpvweig4(Thetas,vert)
% % for a column vector 
% Theta1=Theta1';
% Theta2=Theta2';
% Thetas=[Theta1; Theta2]';
% [musa]=lpvweig4(Thetas,vert)
% 
% % plotting
% figure(1);
% clf();
% t=1:size(musa,1);
% subplot(211)  
% plot(t,Thetas)
% legend('Parameter varying')
% subplot(212)
% plot(t,musa)
% legend('Weighting functions')
% See also
%   piuiobsv darobsv96  
% Author
% Francisco-Ronay López-Estrada (ronaystein@gmail.com)
% Bibliography
% [1] ﻿M., Hamdi., Rodrigues, M., Mechmeche, C., Theilliol, D., Braiek, N. B., & Tunisie, E. P. D. (2009). State Estimation for Polytopic LPV Descriptor Systems :
% Application to Fault Diagnosis. Convergence (pp. 438-443).
n=size(Theta,1);
m=size(Theta,2);
if n<=2 & m<=2
    a=((vert(1,2)-vert(1,1))*(vert(2,2)-vert(2,1)));
        mu1=((Theta(1)-vert(1,1))*(Theta(2)-vert(2,1)))/a;
        mu2=((Theta(1)-vert(1,1))*(vert(2,2)-Theta(2)))/a;
        mu3=((vert(1,2)-Theta(1))*(Theta(2)-vert(2,1)))/a;
        mu4=((vert(1,2)-Theta(1))*(vert(2,2)-Theta(2)))/a;
        mus=[mu1 mu2 mu3 mu4 ];
        musa=[mus sum(mus)];
        elseif n==2 & m>2

for k=1:m
    a=((vert(1,2)-vert(1,1))*(vert(2,2)-vert(2,1)));
        mu1=((Theta(1,k)-vert(1,1))*(Theta(2,k)-vert(2,1)))/a;
        mu2=((Theta(1,k)-vert(1,1))*(vert(2,2)-Theta(2,k)))/a;
        mu3=((vert(1,2)-Theta(1,k))*(Theta(2,k)-vert(2,1)))/a;
        mu4=((vert(1,2)-Theta(1,k))*(vert(2,2)-Theta(2,k)))/a;
        mus=[mu1 mu2 mu3 mu4 ];
       
        musa(k,:)=[mus sum(mus)];
    
    
end
     elseif  n>2 & m==2
        
for k=1:n
    a=((vert(1,2)-vert(1,1))*(vert(2,2)-vert(2,1)));
        mu1=((Theta(k,1)-vert(1,1))*(Theta(k,2)-vert(2,1)))/a;
        mu2=((Theta(k,1)-vert(1,1))*(vert(2,2)-Theta(k,2)))/a;
        mu3=((vert(1,2)-Theta(k,1))*(Theta(k,2)-vert(2,1)))/a;
        mu4=((vert(1,2)-Theta(k,1))*(vert(2,2)-Theta(k,2)))/a;
        mus=[mu1 mu2 mu3 mu4 ];
       
        musa(k,:)=[mus sum(mus)];
    
    
end  

end