function [N,G,L,Ti,H2,Phi]=piuiobsv(dsys,alpha, solv)

%This function compute the gains for a full order Proportiona-Integral
% observer with unknown inputs and useful for fault detection [1] 
% 
%    [N,G1,L,H2,B1]=piuiobsv(dsys)  % default alpha=1
%    [N,G1,L,H2,B1]=piuiobsv(dsys,alpha)
%    [N,G1,L,H2,B1]=piuiobsv(dsys,alpha, solv)
%     
% Parameters
% 
% E, A, B, C, R:
% Continuos matrices of descriptor system. E singular
% alpha:
% For the stabilize the observer in a stable LMI region (see [1])
% N, G1 L, H2, Ti, Phi:
% Gains matrices of the observer
% 
% 
% Description
% 
%   Given a Descriptor system in the form
%     dxEdt=A x(t) + B u(t) +R d (t)
%      y   =C x(t) + D u(t) 
%  This compute the observer gains for 
%    
%    dz/dt= Nz(t) + Gu(t) + Ly(t)+Tide(t)
%       xe= z(t) + H2y(t)
%      dde/dt= Phi (y(t)-ye(t))  
%   
%  xe = stimated states
%   y(t)=output
%   ye(t)=stimated output
% de= stimated unknown input 
% alpha put the observer in the stability LMI region
%      
% Examples
% Descriptor System
% 
% E=[1 0 0 0; 0 1 0 0; 0 0 0 0; 0 0 0 0];
% A=[-0.75 1 0 0; -1 -0.85 0 0; 0 -1 -0.75 0; 0 0 0 -1];
% B=[1 10 ;1 0.5 ; 0 0  ;1 0 ];
% % is very important select a correct matrix C
% % that meets the observability condition
% C=[1 0 1 0; 0 1 0 1; 1 0 0 0]; 
% 
% R=[-1 0 0 0]';
% [dsys]=dsystem(E,A,B,C,R);
% % Observer Gains
% 
% [N,G,L,Ti,H2,Phi]=piuiobsv(dsys,1);
% 
%   
% See also
%   piuiobsv darobsv96  
% 
% Authors
% Francisco Ronay LÓPEZ ESTRADA (ronaystein@gmail.com)
%D. THEILLIOL
%C. M. ASTORGA-ZARAGOZA
%J. C. PONSART 
% 
% 
% Bibliography
% [1] ﻿M., Hamdi., Rodrigues, M., Mechmeche, C., Theilliol, D., Braiek, N. B., & Tunisie, E. P. D. (2009). State Estimation for Polytopic LPV Descriptor Systems : Application to Fault Diagnosis. Convergence (pp. 438-443).

[E,A,B,C,R]=dsys2ss(dsys);

in=nargin();
if in<2 
    solv='lmilab'
    alpha=1;
    
elseif in<3
solv='lmilab' % default solver for the LMI
end

n=size(E,1);
  m=size(C,1);
  
%% cheking the feasibility of the observer
 [nC,Co_mat,nR,Ro_mat,nI,I_mat]=dobsv(E,A,C);
if rank(C)==m
    if nR.Obsv=='T'
%         if nI.Obsv=='T'
%         end
    end
else
    error ('The system does not meet the conditions of R or I Observability ');
end
    
EC=pinv([E;C]);
H1=EC(:,1:n);
H2=EC(:,n+1:end);

    %%eq 18-21
    Ti=H1*R;
    G=H1*B;
    %%Eq 32+
    aux=[H1*A  Ti];
    l=size(aux,2);
    As=[aux; zeros(1,l)];
Cs=[C zeros(size(C,1),1)];



  yalmip ('clear')
Q=sdpvar(l,l,'symmetric');
Wi=sdpvar(l,size(C,1),'full');

% %%eq. 29
%  
F=[Q>0];
F=[F, As'*Q+Q*As-Cs'*Wi'-Wi*Cs+2*alpha*Q<0  ];

ops = sdpsettings('solver',solv,'verbose',0);
solvesdp(F,-trace(Q),ops);
Q=double(Q);
W=double(Wi);



Kis=inv(Q)*W;
% load Kis.mat;
K=Kis(1:n,:);
Phi=Kis(n+1,:);
N=H1*A-K*C;
L=K+N*H2;
